package com.fastbee.system.service.impl;

import java.util.*;
import java.util.stream.Collectors;


import cn.hutool.http.HttpUtil;
import cn.hutool.json.JSONArray;
import cn.hutool.json.JSONException;
import cn.hutool.json.JSONObject;
import cn.hutool.json.JSONUtil;
import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.extension.conditions.query.LambdaQueryChainWrapper;
import com.fastbee.common.exception.ServiceException;
import org.apache.commons.lang3.StringUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import com.fastbee.system.mapper.SysDistrictMapper;
import com.fastbee.system.domain.SysDistrict;
import com.fastbee.system.service.ISysDistrictService;


/**
 * 行政区划Service业务层处理
 * 
 * @author kerwincui
 * @date 2024-10-18
 */
@Service
public class SysDistrictServiceImpl implements ISysDistrictService 
{
    @Autowired
    private SysDistrictMapper sysDistrictMapper;

    /**
     * 查询行政区划
     * 
     * @param id 行政区划主键
     * @return 行政区划
     */
    @Override
    public SysDistrict selectSysDistrictById(Long id)
    {
        return sysDistrictMapper.selectSysDistrictById(id);
    }

    /**
     * 查询行政区划列表
     * 
     * @param sysDistrict 行政区划
     * @return 行政区划
     */
    @Override
    public List<SysDistrict> selectSysDistrictList(SysDistrict sysDistrict)
    {
        return sysDistrictMapper.selectSysDistrictList(sysDistrict);
    }

    /**
     * 新增行政区划
     * 
     * @param sysDistrict 行政区划
     * @return 结果
     */
    @Override
    public int insertSysDistrict(SysDistrict sysDistrict)
    {
        return sysDistrictMapper.insertSysDistrict(sysDistrict);
    }

    /**
     * 修改行政区划
     */
    @Override
    public int updateSysDistrict(SysDistrict sysDistrict)
    {
        String mapOutlineUrl = sysDistrict.getMapOutlineUrl();
        //未上传行政区划轮廓不做处理
        if(StringUtils.isBlank(mapOutlineUrl)){
            return sysDistrictMapper.updateSysDistrict(sysDistrict);
        }

        //判断行政区划轮廓json文件路url是否是完整url还是相对url
        if((mapOutlineUrl.contains("http://") || mapOutlineUrl.contains("https://"))){
            //如果不是是相对url则代表修改过行政区划轮廓数据
            String jsonData = HttpUtil.get(mapOutlineUrl);
            System.err.println("获取到行政区划轮廓json:"+jsonData);
            //读取文件内容
            if (Objects.isNull(jsonData)) {
                throw new ServiceException("上传行政区划地图轮廓json数据为空！");
            }
            //校验文件内容是否是json格式
            if (!isValidJson(jsonData)) {
                throw new ServiceException("json格式不正确！");
            }
            int startIndex = mapOutlineUrl.indexOf("/profile/upload/");
            String relativeUrl = mapOutlineUrl.substring(startIndex);
            sysDistrict.setMapOutlineUrl(relativeUrl);
            sysDistrict.setMapOutline(jsonData);
        }

        return sysDistrictMapper.updateSysDistrict(sysDistrict);
    }

    /**
     * 验证字符串是否是json格式
     */
    public  boolean isValidJson(String jsonStr) {
        try {
            JSONUtil.parseObj(jsonStr);
        } catch (JSONException e) {
            return false;
        }
        return true;
    }

    /**
     * 批量删除行政区划
     */
    @Override
    public int deleteSysDistrictByIds(Long[] ids)
    {
        return sysDistrictMapper.deleteSysDistrictByIds(ids);
    }

    /**
     * 删除行政区划信息
     */
    @Override
    public int deleteSysDistrictById(Long id)
    {
        return sysDistrictMapper.deleteSysDistrictById(id);
    }

    /**
     * 查询行政区划树状列表
     */
    @Override
    public List<SysDistrict> selectSysDistrictTree(Integer level) {
        System.err.println("等级:level:"+level);
        //计算查询用时
        long startTime = System.currentTimeMillis();
        List<SysDistrict> sysDistrictList = new LambdaQueryChainWrapper<>(sysDistrictMapper)
                .select(SysDistrict::getId,SysDistrict::getAdcode, SysDistrict::getParentId, SysDistrict::getName,
                        SysDistrict::getFullName )
                .ge(Objects.nonNull(level),SysDistrict::getLevel, level)
                .list();
        System.err.println("查询用时:"+(System.currentTimeMillis()-startTime));
        return buildTree(sysDistrictList);
    }
    /**
     * 查询指定地区行政区划树状列表
     */
    @Override
    public List<SysDistrict> selectSysDistrictTreeByCode(Long areaCode) {
        LambdaQueryChainWrapper<SysDistrict> sysDistrictLambdaQueryChainWrapper = new LambdaQueryChainWrapper<>(sysDistrictMapper)
                .select(SysDistrict::getId, SysDistrict::getParentId, SysDistrict::getName, SysDistrict::getAdcode,
                        SysDistrict::getLevel, SysDistrict::getFullName, SysDistrict::getName
                        , SysDistrict::getLng, SysDistrict::getLat);
        //获取子级别
        List<SysDistrict> sysDistrictList = sysDistrictLambdaQueryChainWrapper.eq(SysDistrict::getAdcode, areaCode)
                .list();
        if (Objects.isNull(sysDistrictList) || sysDistrictList.isEmpty()) {
            throw new ServiceException("未查询到指定地区行政区划树状列表！");
        }

        //查询全部
        List<SysDistrict> sysDistrictListAll =  new LambdaQueryChainWrapper<>(sysDistrictMapper)
                .select(SysDistrict::getId, SysDistrict::getParentId, SysDistrict::getName, SysDistrict::getAdcode,
                        SysDistrict::getLevel, SysDistrict::getFullName, SysDistrict::getName
                        , SysDistrict::getLng, SysDistrict::getLat)
                .ge(SysDistrict::getAdcode, areaCode)
                .lt(SysDistrict::getAdcode, (Long.parseLong(areaCode.toString().substring(0,2))+1)*10000000000L)
                .list();
        System.err.println("查询到全部列表上限条件");
        System.err.println(((Long.parseLong(areaCode.toString().substring(0,2))+1)*10000000000L));
        List<SysDistrict> sysDistricts = buildTree(sysDistrictListAll);
        return Collections.singletonList(sysDistricts.get(0));
    }



    private List<SysDistrict> buildTree(List<SysDistrict> SysDistricts) {
        Map<Long, SysDistrict> map = new HashMap<>();
        List<SysDistrict> roots = new ArrayList<>();
        for (SysDistrict sysDistrict : SysDistricts) {
            map.put(sysDistrict.getAdcode(), sysDistrict);
        }
        return getDistricts(SysDistricts, map, roots);
    }

    private List<SysDistrict> getDistricts(List<SysDistrict> SysDistricts, Map<Long, SysDistrict> map, List<SysDistrict> roots) {
        for (SysDistrict sysDistrict : SysDistricts) {
            Long parentId = sysDistrict.getParentId();
            if (parentId == null || !map.containsKey(parentId)) {
                roots.add(sysDistrict);
            } else {
                SysDistrict parent = map.get(parentId);
                if (parent.getChildren() == null) {
                    parent.setChildren(new ArrayList<>());
                }
                parent.getChildren().add(sysDistrict);
            }
        }
        return roots;
    }

}
